
# require "eitil_integrate/application_exporter/log_state"

module EitilIntegrate::RubyXL
    class ApplicationExporter

      private

      def log_state 

        return unless write_log == true

        # create_log_sheet
        book.add_worksheet('log')

        # manage sheets
        previous_sheet = @sheet.sheet_name
        @sheet         = @book["log"]

        # manage coordinates
        previous_x = @x
        @x         = 0

        # log everything we want to log
        report_state

        # style logs
        base_style_first_x_columns_width 1, 40
        style_first_column_bold

        # restore what was previously active
        @sheet = @book[previous_sheet]
        @x     = previous_x

      end

      def report_state
        instance_variables.each do |ivar|

          variable_name  = ivar.to_s
          variable_value = format_value(instance_variable_get(ivar))

          array_to_row [variable_name, variable_value]

        end
      end

      # formatting the values to a human readably format is important, because otherwise Excel warns
      # users on safety when opening the file.
      def format_value(value)
        value_class = value.class

        if value_class == Hash
          value.map {|k,v| "#{k.to_s.split('_').map(&:capitalize).join(' ')}: #{v}"}.join(' // ')

        elsif value_class == Array && value.all? { |item| item.class.superclass == ApplicationRecord }
          "#{value.class} #{value.map(&:id).join(', ')}"

        elsif value_class.superclass == ApplicationRecord
         "#{value.class} ##{value.id}"

        elsif value_class.superclass == ActiveRecord::Relation
          "#{value.class.to_s.split('::').first} #{value.ids.to_s.delete('[]')}"

        elsif value_class.superclass == ActiveRecord::Associations::CollectionProxy
          "#{value.class.to_s.split('::').first} #{value.ids.to_s.delete('[]')}"

        elsif value_class == RubyXL::Workbook
          value_class.name

        elsif value_class == RubyXL::Worksheet
          value_class.name

        else
          value.to_s

        end
      end

  end
end